; AisleRiot - bristol.scm
; Copyright (C) 1999 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
  (initialize-playing-area)
  (set-ace-low)
  (make-standard-deck)
  (shuffle-deck)
	(set-deal-count 3)

  (add-slots)

	(set! stock 0)
	(set! reserve '(1 2 3))
  (set! foundations '(4 5 6 7))
  (set! tableau '(8 9 10 11 12 13 14 15))

  (deal-cards-face-up stock (append tableau tableau tableau))
  (map check-for-kings tableau)

  (give-status-message)

  (list 10 3)
)

(define (add-slots)
	(define card-tq-width (quotient (* 3 (get-card-width)) 4))
	(define (add-wide-extended-slot)
		(add-extended-slot '() right)
		(add-blank-slot)
		(set! HORIZPOS (+ HORIZPOS card-tq-width))
	)

	(add-normal-slot DECK)

	(set! HORIZPOS (+ HORIZPOS card-tq-width))

	; reserve
	(add-normal-slot '())
	(add-normal-slot '())
	(add-normal-slot '())

	(set! HORIZPOS (+ HORIZPOS card-tq-width))
	; foundations
	(add-normal-slot '())
	(add-normal-slot '())
	(add-normal-slot '())
	(add-normal-slot '())

	(add-carriage-return-slot)

	(add-wide-extended-slot)
	(add-wide-extended-slot)
	(add-wide-extended-slot)
	(add-extended-slot '() right)

	(add-carriage-return-slot)

	(add-wide-extended-slot)
	(add-wide-extended-slot)
	(add-wide-extended-slot)
	(add-extended-slot '() right)
)

(define (give-status-message)
  (set-statusbar-message (get-stock-no-string)))

(define (check-for-kings slot-id)
	(define card-list (get-cards slot-id))
	(set-cards! slot-id
		(if (= king (get-value (car card-list)))
			(if (= king (get-value (cadr card-list)))
				(list (caddr card-list) (car card-list)
					(cadr card-list))
				(list (cadr card-list) (caddr card-list)
					(car card-list))
			)
			(if (= king (get-value (cadr card-list)))
				(list (car card-list) (caddr card-list)
					(cadr card-list))
				card-list
			)
		)
	)
)

(define (button-pressed slot-id card-list)
	(and
		(not (empty-slot? slot-id))
		(member slot-id (append reserve tableau))
		(= (length card-list) 1)
	)
)

(define (button-released start-slot card-list end-slot)
	(define card (car card-list))
	(if (foundation? end-slot)
		(and
			(if (empty-slot? end-slot)
				(= ace (get-value card))
				(card-follows? card (get-top-card end-slot))
			)
			(add-to-score! 1)
			(move-n-cards! start-slot end-slot card-list)
		)
		(and
			(member end-slot tableau)
			(not (empty-slot? end-slot))
			(card-follows? card (get-top-card end-slot))
			(move-n-cards! start-slot end-slot card-list)
		)
	)
)

(define (button-clicked slot-id)
	(and
		(= slot-id 0)
		(not (empty-slot? 0))
		(if (> (length (get-cards slot-id)) 3)
			(deal-cards-face-up 0 reserve)
			(deal-cards-face-up 0 '(1))
		)
	)
)

(define (find-foundation-slot card)
	(define (f slot)
		(and
			(not (empty-slot? slot))
			(card-follows? (get-top-card slot) card)
			slot
		)
	)
	(if (= ace (get-value card))
		(find-empty-slot foundations)
		(or-map f foundations)
	)
)

(define (button-double-clicked slot-id)
	(and
		(member slot-id (append reserve tableau))
		(let ((dest (find-foundation-slot (get-top-card slot-id))))
			(and
				dest
				(deal-cards slot-id (list dest))
				(add-to-score! 1)
			)
		)
	)
)

(define (game-continuable)
	(give-status-message)
	(and
		(not (game-won))
		(get-hint)
	)
)

(define (game-won)
	(foundations-full?)
)

(define (check-reserve slot-id)
	(define (f slot2)
		(check-a-tslot slot-id (get-cards slot-id) 1 slot2)
	)
	(and
		(not (empty-slot? slot-id))
		(or-map f tableau)
	)
)

(define (deepness card-list depth)
	(if (and	(> (length card-list) 1)
			(card-follows? (car card-list) (cadr card-list)))
		(deepness (cdr card-list) (+ 1 depth))
		depth
	)
)

(define (check-a-tslot slot1 card-list depth slot2)
	(define (f slot3)
	  	(check-a-tslot slot1 (cdr card-list) (- depth 1) slot3)
	)
	(and
		(not (and (= (length card-list) depth) (> slot1 7)))
		(not (empty-slot? slot2))
		(not (= slot1 slot2))
		(card-follows? (car card-list) (get-top-card slot2))
	  	(or
			(= depth 1)
			(or-map f tableau)
		)
		(list 1 (get-name (get-top-card slot1))
			(get-name (get-top-card slot2)))
	)
)

(define (check-tableau slot-id)
	(define depth (deepness (get-cards slot-id) 1))
	(define (f slot2)
		(check-a-tslot slot-id (get-cards slot-id) depth slot2)
	)
	(and
		(not (empty-slot? slot-id))
		(or-map f tableau)
	)
)

(define (get-hint)
	(or
		(or-map check-to-foundation (append reserve tableau))
		(or-map check-reserve reserve)
		(or-map check-tableau tableau)
		(deal-hint)
	)
)

(define (get-options) 
  #f)

(define (apply-options options) 
  #f)

(define (timeout) 
  #f)

(set-lambda new-game button-pressed button-released button-clicked
button-double-clicked game-continuable game-won get-hint get-options
apply-options timeout)
