; AisleRiot - camelot.scm
; Copyright (C) 1998 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define add-stage #t)

(define jack-slots '(4 7 8 11))
(define queen-slots '(1 2 13 14))
(define king-slots '(0 3 12 15))
(define centre-slots '(5 6 9 10))

(define (new-game)
	(initialize-playing-area)
	(set-ace-low)
	(make-standard-deck)
	(shuffle-deck)
	(set-redeals 0)

	(add-slots)

	(set! tableau (append jack-slots queen-slots king-slots centre-slots))
	(set! stock 16)
	(set! waste 17)
	(set! add-stage #t)

	(give-status-message)

	(list 7 4)
)

(define (add-slots)
	(define card-half-width (quotient (get-card-width) 2))
	(define (add-row)
		(add-blank-slot)
		(add-blank-slot)
		(set! HORIZPOS (+ HORIZPOS card-half-width))
		(add-normal-slot '())
		(add-normal-slot '())
		(add-normal-slot '())
		(add-normal-slot '())
		(add-carriage-return-slot)
	)

	(add-row)
	(add-row)
	(add-row)
	(add-row)

	(set! HORIZPOS (get-horiz-start))
	(set! VERTPOS (get-vert-start))

	(add-normal-slot DECK)
	(add-normal-slot '())
)

(define (give-status-message)
	(set-statusbar-message (get-stock-no-string))
)

(define (button-pressed slot-id card-list)
  (and (not (empty-slot? slot-id))
       (if (= slot-id 17)
	   (and
	   	(or-map empty-slot? tableau)
		(set! add-stage #t) 
	   )
	   (not (or add-stage
		    (= slot-id 16)
		    (= (get-value (car card-list)) king)
		    (= (get-value (car card-list)) queen)
		    (= (get-value (car card-list)) jack))))))

(define (button-released start-slot card-list end-slot)
  (if add-stage
      (and (empty-slot? end-slot) 
	   (cond ((= (get-value (car card-list)) king)
		  (member end-slot king-slots))
		 ((= (get-value (car card-list)) queen)
		  (member end-slot queen-slots))
		 ((= (get-value (car card-list)) jack)
		  (member end-slot jack-slots))
		 (#t (not (= end-slot 16))))
	   (move-n-cards! start-slot end-slot card-list)
	   #t)
      (if (= start-slot end-slot)
	  (and (= (get-value (car card-list)) 10)
		(add-to-score! 1))	       
	  (and (not (empty-slot? end-slot))
	       (not (= end-slot 17))
	       (= 10 (+ (get-value (car card-list))
			(get-value (car (get-cards end-slot)))))
	       (remove-card end-slot)
		(add-to-score! 2)))))

(define (button-clicked slot-id)  
  (if (= slot-id 16)
      (and (empty-slot? 17)
	   (or-map empty-slot? tableau)
	   (set! add-stage #t)
	   (flip-stock)
      (and (not add-stage)
	   (not (empty-slot? slot-id))
	   (is-visible? (get-top-card slot-id))
	   (= 10 (get-value (get-top-card slot-id)))
	   (add-to-score! 1)
	   (remove-card slot-id))))

(define (button-double-clicked slot)
  #f)     

(define (game-won)
	(and-map empty-slot? (cons stock (cons waste centre-slots)))
)

(define (get-top-cards)
	(define (f slot cards)
		(if (= slot 16)
			cards
			(f (+ 1 slot) (if (empty-slot? slot)
				cards
				(cons (get-top-card slot) cards)
			))
		)
	)
	(f 0 '())
)

(define (find-card-val-in-list? cards value)
  (and (not (null? cards))
       (if (= value (get-value (car cards))) 
	   (car cards)
	   (find-card-val-in-list? (cdr cards) value))))

(define (find-match cards)
  (and (not (null? cards))
       (if (= 10 (get-value (car cards))) 
	   (list 2 (get-name (car cards)) "itself") ; yuk..
	   (let ((match (find-card-val-in-list? 
			 (cdr cards)
			 (- 10 (get-value (car cards))))))
	     (if match
		 (list 1 (get-name (car cards)) (get-name match))
		 (find-match (cdr cards)))))))

(define (placeable? card)
  (cond ((= (get-value card) king)
	 (and (or-map empty-slot? king-slots)
	      "an empty corner slot"))
	 ((= (get-value card) queen)
	  (or (and (or (empty-slot? 1)
		       (empty-slot? 2))
		   "an empty top slot")
	      (and (or (empty-slot? 13)
		       (empty-slot? 14))
		   "an empty bottom slot")))
	((= (get-value card) jack)
	  (or (and (or (empty-slot? 4)
		       (empty-slot? 8))
		   "an empty left slot")
	      (and (or (empty-slot? 7)
		       (empty-slot? 11))
		   "an empty right slot")))
	(#t "an empty slot")))

(define (game-over)
	(give-status-message)
	(if (or (not (or-map empty-slot? tableau))
			(and (empty-slot? 16) (empty-slot? 17)))
		(begin
			(set! add-stage #f)
			(find-match (get-top-cards))
		)
		(or
			(empty-slot? 17)
			(placeable? (get-top-card 17))
		)
	)
)

(define (get-hint)
	(or
		(if add-stage
			(and
				(not (empty-slot? 17))
				(list 2 (get-name (get-top-card 17))
					(placeable? (get-top-card 17)))
			)
			(find-match (get-top-cards))
		)
		(deal-hint)
      )
)

(define (undo-func data)
  (set-score! (car data))
  (set! add-stage (cadr data)))

(define (record-move slot-id old-cards)
  (set! MOVE (list undo-func
		   (list (get-score) add-stage)
		   (snapshot-board 0 slot-id old-cards))))

(define (get-options) #f)

(define (apply-options options) #f)

(define (timeout) #f)

(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
