; AisleRiot - clock.scm
; Copyright (C) 1998 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define slot-values #(11 12 1 10 2 9 13 3 8 4 7 6 5))

(define (new-game)
  (initialize-playing-area)
  (make-standard-deck)
  (shuffle-deck)

  (add-three-row)
  (add-two-row)

  (add-extended-slot '() right)
  (add-blank-slot)
  (add-blank-slot)
  (add-extended-slot DECK right)
  (add-blank-slot)
  (add-blank-slot)
  (add-extended-slot '() right)
  (add-carriage-return-slot)

  (add-two-row)
  (add-three-row)

  (set! tableau '(2 4 7 9 12 11 10 8 5 3 0 1))

  (deal-cards 6 (append tableau tableau tableau tableau))

  (flip-top-card 6)

  (list 9 5)
)

(define (add-three-row)
	(add-blank-slot)
	(add-extended-slot '() right)
	(add-blank-slot)
	(add-extended-slot '() right)
	(add-blank-slot)
	(add-extended-slot '() right)
	(add-carriage-return-slot)
)

(define (add-two-row)
	(add-extended-slot '() right)
	(add-blank-slot)
	(add-blank-slot)
	(add-blank-slot)
	(add-blank-slot)
	(add-blank-slot)
	(add-extended-slot '() right)
	(add-carriage-return-slot)
)

(define (button-pressed slot-id card-list)
  (and (= slot-id 6)
       (= (length card-list) 1)))

(define (transaction-good? end-slot card-list)
	(= (vector-ref slot-values end-slot) (get-value (car card-list)))
)

(define (complete-transaction card-list end-slot)
	(add-cards! end-slot card-list)
	(add-card! 6 (get-bottom-card end-slot))
	(set-cards! end-slot (reverse (cdr (reverse (get-cards end-slot)))))
	(make-visible-top-card 6)
	(and
		(not (= end-slot 6))
		(add-to-score! 1)
	)
	#t
)

(define (button-released start-slot card-list end-slot)
	(and
		(transaction-good? end-slot card-list)
		(complete-transaction card-list end-slot)
	)
)

(define (button-clicked slot-id)
	(and
		(= (get-value (get-top-card slot-id)) king)
		(is-visible? (get-top-card slot-id))
		(set-cards! 6 (cons (car (reverse (get-cards 6)))
			    (reverse (cdr (reverse (get-cards 6))))))
		(make-visible-top-card 6)
	)
)

(define (button-double-clicked slot)
	(and
		(not (= 6 slot)) ; click already took care of this
		(transaction-good? slot (get-cards 6))
		(complete-transaction (list (remove-card 6)) slot)
	)
)

(define (make-slot-visible slot)
	(map make-visible (get-cards slot))
)


(define (game-won)
	(define (f slot)
		(define val (vector-ref slot-values slot))
		(and-map (lambda (c) (= val (get-value c))) (get-cards slot))
	)
	(map make-slot-visible tableau)
	(and-map f tableau)
)

(define (game-over)
	(not (and
		(is-visible? (get-bottom-card 6))
		(= (get-value (get-top-card 6)) king)
		(map make-slot-visible tableau)
	))
)

(define hint-vector #(
	"Just because a crosswalk looks like a hopscotch board doesn't mean it is one" 
	"Look both ways before you cross the street"
	"Have you read the help file?"
	"Odessa is a better game.  Really."
	"Tourniquets are not recommended unless in the direst emergency"
	"I could sure use a backrub right about now..."
	"Monitors won't give you Vitamin E -- but sunlight will..."
	"If you're ever lost and alone in the woods, hug a tree"
	"Fishing wire makes bad dental floss"
	"Consistency is key"
	"When without a stapler, a staple and a ruler will work"
	"Never blow in a dog's ear"))

(define (get-hint)
	(list 0 (vector-ref hint-vector (random 12)))
)

(define (get-options) #f)

(define (apply-options options) #f)

(define (timeout) #f)

(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
