; AisleRiot - doublets.scm
; Copyright (C) 1998 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA


(define (new-game)
	(initialize-playing-area)
	(set-ace-low)
	(make-standard-deck)
	(shuffle-deck)
	(set-redeals 2)

	(add-slots)

	(set! stock 0)
	(set! waste 1)
	(set! tableau '(2 3 4 5 6 7 9))
	(set! foundations '(8))

  (deal-cards-face-up 0 '(2 3 4 5 6 7 8 9))

  (check-kings '(2 3 4 5 6 7 8 9))

  (add-to-score! 1)

  (give-status-message)

  (list 6 3)
)

(define (add-slots)
	(add-normal-slot DECK)
	(add-normal-slot '())

	(add-blank-slot)
	(add-normal-slot '())
	(add-normal-slot '())
	(add-normal-slot '())
	(add-carriage-return-slot)

	(add-blank-slot)
	(add-blank-slot)
	(add-blank-slot)
	(add-normal-slot '())
	(add-blank-slot)
	(add-normal-slot '())
	(add-carriage-return-slot)

	(add-blank-slot)
	(add-blank-slot)
	(add-blank-slot)
	(add-normal-slot '())
	(add-normal-slot '())
	(add-normal-slot '())
)

(define (give-status-message)
	(set-statusbar-message (string-append (get-stock-no-string) "   "
		(get-redeals-string)))
)

(define (check-kings slot-list)
	(define slot (car slot-list))
	(if (= (get-value (get-top-card slot)) king)
		(begin
			(set-cards! 0 (reverse (cons
				(flip-card (remove-card slot))
				(reverse (get-cards 0))
			)))
			(deal-cards-face-up 0 (list slot))
			(check-kings slot-list)
		)
		(and
			(> (length slot-list) 1)
			(check-kings (cdr slot-list))
		)
	)
)

(define (button-pressed slot-id card-list)
	(and
		(not (empty-slot? slot-id))
		(not (= slot-id 0))
		(not (= slot-id 8))
		(not (= (get-value (car card-list)) king))
	)
)

(define (wanted)
	(modulo (* 2 (get-value (get-top-card 8))) 13)
)

(define (next-card? card)
	(= (get-value card) (wanted))
)

(define (replace-cards slot)
	(cond
		(	(not (empty-slot? 1))
			(deal-cards 1 (list slot))
		)
		(	(and	(not (empty-slot? 0))
				(not (= slot 1))
			)
			(deal-cards-face-up 0 (list slot))
		)
	)
	#t
)

(define (button-released start-slot card-list end-slot)
	(and	(= end-slot 8)
		(next-card? (car card-list))
		(add-to-score! 1)
		(move-n-cards! start-slot end-slot card-list)
		(replace-cards start-slot)
	)
)

(define (button-clicked slot-id)
	(if (= slot-id stock)
		(flip-stock)
	)
)

(define (button-double-clicked slot)
	(and	(not (empty-slot? slot))
		(not (= slot 0))
		(next-card? (get-top-card slot))
		(add-to-score! 1)
		(deal-cards slot '(8))
		(replace-cards slot)
	)
)

(define (check-move slot)
	(or
		(and
			(not (empty-slot? slot))
			(next-card? (get-top-card slot))
		)
		(and
			(< slot 9)
			(check-move (+ 1 slot))
		)
	)
)

(define (game-over)
	(give-status-message)
	(not (or
		(game-won)
		(and
			(not (deal-hint))
			(not (check-move 1))
		)
	))
)

(define (game-won)
	(= 48 (length (get-cards 8)))
)

(define (get-hint)
	(list 4 (get-value-name (wanted)))
)

(define (get-options) #f)

(define (apply-options options) #f)

(define (timeout) #f)

(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
