; AisleRiot - eagle_wing.scm
; Copyright (C) 1998 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
  (initialize-playing-area)
  (set-ace-low)
  (make-standard-deck)
  (shuffle-deck)
	(set-redeals 2)

  (add-normal-slot DECK)
  (add-normal-slot '())

  (add-blank-slot)
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (set! foundations '(2 3 4 5))

  (add-carriage-return-slot)
  (set! VERTPOS (+ VERTPOS 100))
  (add-extended-slot '() down)          ;tableau (slot 6)
  (set! VERTPOS (- VERTPOS 20))
  (add-extended-slot '() down)          ;tableau (slot 7)
  (set! VERTPOS (- VERTPOS 20))
  (add-extended-slot '() down)          ;tableau (slot 8)
  (set! VERTPOS (+ VERTPOS 20))
  (add-extended-slot '() down)          ;tableau (slot 9)
  (set! VERTPOS (+ VERTPOS 50))
  (add-normal-slot '())                 ;reserve (slot 10)
  (set! VERTPOS (- VERTPOS 50))
  (add-extended-slot '() down)          ;tableau (slot 11)
  (set! VERTPOS (- VERTPOS 20))
  (add-extended-slot '() down)          ;tableau (slot 12)
  (set! VERTPOS (+ VERTPOS 20))
  (add-extended-slot '() down)          ;tableau (slot 13)
  (set! VERTPOS (+ VERTPOS 20))
  (add-extended-slot '() down)          ;tableau (slot 14)

	(set! stock 0)
	(set! waste 1)
	(set! reserve 10)
	(set! tableau '(6 7 8 9 11 12 13 14))
	(set! moveable (cons waste (cons reserve tableau)))

  (deal-cards-face-up 0 (list reserve))
  (deal-cards 0 '(10 10 10 10 10 10 10 10 10 10 10 10))
  (deal-cards-face-up 0 (cons 2 tableau))

  (add-to-score! 1)
  (set-base-card! (get-value (get-top-card 2)))

  (give-status-message)

  (list 9 3)
)

(define (give-status-message)
	(set-statusbar-message (string-append (get-stock-no-string) "   "
		(get-reserve-no-string) "   " (get-base-string) "   "
		(get-redeals-string)))
)

(define (button-pressed slot-id card-list)
	(and
		(not (foundation? slot-id))
		(is-visible? (car card-list))
	)
)

(define (complete-transaction start-slot card-list end-slot)
	(if (foundation? end-slot)
		(add-to-score! (length card-list)))
	(move-n-cards! start-slot end-slot card-list)
	(and
		(not (= start-slot waste))
		(empty-slot? start-slot)
		(not (empty-slot? reserve))
		(deal-cards-face-up reserve (list start-slot))
	)
	(give-status-message)
)

(define (button-released start-slot card-list end-slot)
  (and (not (= start-slot end-slot))
       (or (and (foundation? end-slot)
		(next-foundation-card? (car card-list) end-slot)
		(complete-transaction start-slot (reverse card-list) end-slot))
	   (and (member end-slot tableau)
		(= (length card-list) 1)
		(or (empty-slot? end-slot)
		    (and (< (length (get-cards end-slot)) 3)
			 (suit-next? (car card-list) (get-top-card end-slot))))
		(complete-transaction start-slot card-list end-slot)))))

(define (button-clicked slot-id)
	(and
		(= slot-id 0)
		(begin
			(flip-stock)
			(give-status-message)
		)
	)
)

(define (button-double-clicked slot)
	(and
		(not (empty-slot? slot))
		(let* (	(card (get-top-card slot))
			(end-slot (find-foundation-slot card))
			)
			(and
				end-slot
				(is-visible? card)
				(remove-card slot)
				(complete-transaction slot (list card) end-slot)
			)
		)
	)
)

(define (game-over)
  (and (not (game-won))
       (get-hint)))

(define (game-won)
	(foundations-full?)
)

(define (check-empty-slot slot)
	(and
		(not (empty-slot? waste))
		(empty-slot? slot)
		(list 2 (get-name (get-top-card waste)) "an empty tableau slot")
	)
)

(define (check-tableau slot)
	(define card (get-top-card slot))
	(define (f check-slot)
		(and
			(not (= slot check-slot))
			(not (empty-slot? check-slot))
			(< (length (get-cards check-slot)) 3)
			(suit-next? card (get-top-card check-slot))
			(list 1 (get-name card)
				(get-name (get-top-card check-slot)))
		)
	)
	(and
		(not (empty-slot? slot))
		(is-visible? (get-top-card slot))
		(or-map f tableau)
	)
)

(define (get-hint)
	(or
		(or-map check-to-foundation moveable)
		(or-map check-empty-slot tableau)
		(or-map check-tableau moveable)
		(deal-hint)
	)
)

(define (get-options) #f)

(define (apply-options options) #f)

(define (timeout) #f)

(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
