; AisleRiot - elevator.scm
; Copyright (C) 1999 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
  (initialize-playing-area)
  (set-wrap)
  (make-standard-deck)
  (shuffle-deck)

	(add-slots)

	(set! stock 0)
	(set! waste 1)
	(set! tableau '(
	 2
	 3  4
	 5  6  7
	 8  9 10 11
	12 13 14 15 16
	17 18 19 20 21 22
	23 24 25 26 27 28 29))


  (deal-cards 0 tableau)
  (map flip-top-card '(23 24 25 26 27 28 29))

  (give-status-message)

  (list 7 4)
)

(define (add-slots)
	(define card-half-width (quotient (get-card-width) 2))

	(add-normal-slot DECK)
	(add-normal-slot '())
	(add-blank-slot)
	(add-normal-slot '())

	(do
		((i 2 (+ 1 i)))
		((= 8 i))
		(add-carriage-return-slot)
		(set! VERTPOS (- VERTPOS 80))
		(do
			((j (quotient (- 7 i) 2) (- j 1)))
			((>= 0 j))
			(add-blank-slot)
		)
		(if (even? i)
			(set! HORIZPOS (+ HORIZPOS card-half-width))
		)
		(do
			((j 0 (+ 1 j)))
			((= i j))
			(add-normal-slot '())
		)
	)
)

(define (give-status-message)
  (set-statusbar-message (get-stock-no-string)))

(define (button-pressed slot-id card-list)
  (and (is-visible? (car card-list))
       (not (= slot-id 1))))

(define (get-row-number slot)
	(cond
		((< slot 3) 1)
		((< slot 5) 2)
		((< slot 8) 3)
		((< slot 12) 4)
		((< slot 17) 5)
		((< slot 23) 6)
		((< slot 30) 7)
        )
)

(define (check-parents slot)
	(or
		(= slot 23)
		(let ((rowno (get-row-number slot)))
			(and
				(not (empty-slot? slot))
				(empty-slot? (+ slot rowno))
				(empty-slot? (+ slot rowno 1))
				(make-visible-top-card slot)
			)
			#f
		)
		(check-parents (+ 1 slot))
	)
)

(define (button-released start-slot card-list end-slot)
	(and
		(= end-slot 1)
		(not (empty-slot? 1))
		(or
			(card-follows? (get-top-card 1) (car card-list))
			(card-follows? (car card-list) (get-top-card 1))
		)
		(add-to-score! 1)
		(move-n-cards! start-slot end-slot card-list)
		(check-parents 2)
	)
)

(define (button-clicked slot-id)
  (cond ((= slot-id 0)
	 (flip-stock))
	((and (not (= slot-id 1))
	      (not (empty-slot? slot-id))
	      (is-visible? (get-top-card slot-id))
	      (not (empty-slot? 1))
		(or
			(card-follows? (get-top-card 1) (get-top-card slot-id))
			(card-follows? (get-top-card slot-id) (get-top-card 1))
		))
	 (begin
	   (add-to-score! 1)
	   (deal-cards slot-id '(1))
	   (check-parents 2)))
	(#t #f)))

(define (button-double-clicked slot-id)
  (button-clicked slot-id))

(define (playable slot)
	(and
		(not (empty-slot? 1))
		(not (empty-slot? slot))
		(is-visible? (get-top-card slot))
		(or
			(card-follows? (get-top-card 1) (get-top-card slot))
			(card-follows? (get-top-card slot) (get-top-card 1))
		)
		(list 1 (get-name (get-top-card slot))
			(get-name (get-top-card 1))
		)
	)
)

(define (game-continuable)
  (give-status-message)
  (and (not (game-won))
       (get-hint)))

(define (game-won)
  (empty-slot? 2))

(define (get-hint)
	(or
		(or-map playable tableau)
		(deal-hint)
	)
)

(define (get-options) 
  #f)

(define (apply-options options) 
  #f)

(define (timeout) 
  #f)

(set-lambda new-game button-pressed button-released button-clicked
button-double-clicked game-continuable game-won get-hint get-options
apply-options timeout)
