; AisleRiot - kansas.scm
; Copyright (C) 1999 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
	(initialize-playing-area)
	(set-ace-low)
	(make-standard-deck)
	(shuffle-deck)
	(set-redeals 0)

	(add-slots)

	(set! stock 0)
	(set! waste 1)
	(set! foundations '(2 3 4 5))
	(set! reserve 6)
	(set! tableau '(7 8 9))
	(set! moveable (cons reserve (cons waste tableau)))

	(deal-cards 0 '(6 6 6 6 6 6 6 6 6 6 6))
	(deal-cards-face-up 0 '(6 2 7 8 9))

	(set-base-card! (get-value (get-top-card 2)))

	(add-to-score! 1)
	(give-status-message)

	(list 7 4)
)

(define (add-slots)
	(define card-half-width (quotient (get-card-width) 2))

	(add-normal-slot DECK)
	(add-normal-slot '())

	(set! HORIZPOS (+ HORIZPOS card-half-width))

	(add-normal-slot '())
	(add-normal-slot '())
	(add-normal-slot '())
	(add-normal-slot '())

	(add-carriage-return-slot)

	(add-normal-slot '())

	(add-blank-slot)
	(add-blank-slot)

	(set! HORIZPOS (+ HORIZPOS card-half-width))

	(add-extended-slot '() down)
	(add-extended-slot '() down)
	(add-extended-slot '() down)
)

(define (give-status-message)
	(set-statusbar-message (string-append (get-stock-no-string) "   "
		(get-reserve-no-string) "   " (get-base-string)))
)

(define (button-pressed slot-id card-list)
  (and (not (= (length card-list) 0))
       (is-visible? (car card-list))
       (or (= slot-id 1)
	   (> slot-id 5))))

(define (button-released start-slot card-list end-slot)
  (cond ((and (foundation? end-slot)
	      (= (length card-list) 1))
	 (and
	      (next-foundation-card? (car card-list) end-slot)
	      (move-n-cards! start-slot end-slot card-list)
	      (deal-from-reserve start-slot)
	      (give-status-message)))
	((and (> end-slot 6)
	      (empty-slot? end-slot)
	      (= start-slot 1))
	 (move-n-cards! start-slot end-slot card-list))
	((and (> end-slot 6)
	      (not (empty-slot? end-slot)))
	 (and (card-follows? (car (reverse card-list)) (get-top-card end-slot))
	      (move-n-cards! start-slot end-slot card-list)
	      (deal-from-reserve start-slot)
	      (give-status-message)))
	(#t #f)))

(define (button-clicked slot-id)
  (and (= slot-id 0)
       (flip-stock)
       (give-status-message)))

(define (deal-from-reserve slot-id)
	(or
		(= 1 slot-id)
		(empty-slot? 6)
		(and
			(= slot-id 6)
			(make-visible-top-card 6)
		)
		(and
			(empty-slot? slot-id)
			(deal-cards 6 (list slot-id))
			(not (empty-slot? 6))
			(make-visible-top-card 6)
		)
	)
	#t
)

(define (button-double-clicked start-slot)
	(and
		(or
			(> start-slot 5)
			(eq? start-slot 1)
		)
		(not (empty-slot? start-slot))
		(let*	(
				(card (get-top-card start-slot))
				(end-slot (find-foundation-slot card))
			)
			(and
				end-slot
				(remove-card start-slot)
				(move-n-cards! start-slot end-slot (list card))
				(add-to-score! 1)
				(deal-from-reserve start-slot)
				(give-status-message)
			)
		)
	)
)

(define (game-continuable)
  (and (not (game-won))
       (get-hint)))

(define (game-won)
	(foundations-full?)
)

(define (check-a-tableau-list card card-list)
	(and
		(< 0 (length card-list))
		(if (card-follows? (car card-list) card)
			(and
				(or
					(= 1 (length card-list))
					(find-foundation-slot (cadr card-list))
					(suit-eq? card (car card-list))
				)
				(list 1 (get-name (car card-list))
					(get-name card))
			)
			(check-a-tableau-list card (cdr card-list))
		)
	)
)

(define (check-a-tableau card slot2)
	(if (or (= slot2 1) (= slot2 6))
		(and
			(card-follows? (get-top-card slot2) card)
			(list 1 (get-name (get-top-card slot2)) (get-name card))
		)
		(check-a-tableau-list card (get-cards slot2))
	)
)

(define (check-tableau slot-id)
	(define (f dest)
		(and
			(not (empty-slot? dest))
			(check-a-tableau (get-top-card slot-id) dest)
		)
	)
	(and
		(not (empty-slot? slot-id))
		(or-map f moveable)
	)
)

(define (empty-tableau?)
	(and
		(not (empty-slot? 1))
		(or-map empty-slot? tableau)
		(list 2 (get-name (get-top-card 1)) "an empty Tableau slot")
	)
)

(define (get-hint)
	(or
		(or-map check-to-foundation moveable)
		(or-map check-tableau tableau)
		(empty-tableau?)
		(deal-hint)
	)
)

(define (get-options) 
  #f)

(define (apply-options options) 
  #f)

(define (timeout) 
  #f)

(set-lambda new-game button-pressed button-released button-clicked
button-double-clicked game-continuable game-won get-hint get-options
apply-options timeout)
