; AisleRiot - lady_jane.scm
; Copyright (C) 1999 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
  (initialize-playing-area)
  (set-ace-low)
  (make-standard-deck)
  (shuffle-deck)
  (add-slots)
	(set-deal-count 7)

	(set! stock 0)
	(set! waste 1)
	(set! foundations '(2 3 4 5))
	(set! tableau '(6 7 8 9 10 11 12))
	(set! reserve '(13 14 15 16 17 18 19))

  (deal-cards 0 '(7 8 9 10 11 12 8 9 10 11 12 9 10 11 12 10 11 12
		    11 12 12))
  (deal-cards-face-up 0 (append tableau reserve (list 2)))

  (add-to-score! 1)

  (set-base-card! (get-value (get-top-card 2)))

  (give-status-message)

  (list 9 4)
)

(define (add-slots)
	(define card-half-width (quotient (get-card-width) 2))
	(define card-half-height (quotient (get-card-height) 2))
	(define (my-carriage-return)
		(add-carriage-return-slot)
		(set! HORIZPOS (+ HORIZPOS (* 7 (get-card-width)) card-half-width))
	)

  (add-normal-slot DECK)
  (add-normal-slot '())
  
  (add-blank-slot)

  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())

  (add-carriage-return-slot)
 
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)

  (my-carriage-return)
  (set! VERTPOS (+ (get-vert-start) card-half-height))

  (add-normal-slot '())
  (my-carriage-return)

  (add-normal-slot '())
  (my-carriage-return)

  (add-normal-slot '())
  (my-carriage-return)

  (set! VERTPOS (get-vert-start))

  (add-blank-slot)
  (add-normal-slot '())
  (my-carriage-return)

  (add-blank-slot)
  (add-normal-slot '())
  (my-carriage-return)

  (add-blank-slot)
  (add-normal-slot '())
  (my-carriage-return)

  (add-blank-slot)
  (add-normal-slot '())
)

(define (give-status-message)
  (set-statusbar-message (string-append (get-stock-no-string)
					"   "
					(get-base-string))))

(define (button-pressed slot-id card-list)
  (and (not (empty-slot? slot-id))
       (is-visible? (car (reverse card-list)))))

(define (to-tableau? card end-slot)
  (if (empty-slot? end-slot)
      (or (= (get-value card) (- BASE 1))
	  (and (= BASE ace)
	       (= (get-value card) king)))
      (and (not (eq? (is-red? card)
		     (is-red? (get-top-card end-slot))))
	   (not (= (get-value (get-top-card end-slot)) BASE))
	   (or (= (get-value (get-top-card end-slot))
		  (+ 1 (get-value card)))
	       (and (= (get-value (get-top-card end-slot)) ace)
		    (= (get-value card) king))))))

(define (button-released start-slot card-list end-slot)
  (cond ((and (> end-slot 1)
	      (< end-slot 6))
	 (and (= (length card-list) 1)
	      (next-foundation-card? (car card-list) end-slot)
	      (or (and (> start-slot 5)
		       (< start-slot 13)
		       (not (empty-slot? start-slot))
		       (make-visible-top-card start-slot))
		  (and (> start-slot 1)
		       (< start-slot 6)
		       (add-to-score! -1))
		  #t)
	      (add-to-score! 1)
	      (move-n-cards! start-slot end-slot card-list)))
	((and (> end-slot 5)
	      (< end-slot 13))
	 (and (to-tableau? (car (reverse card-list)) end-slot)
	      (or (and (> start-slot 1)
		       (< start-slot 6)
		       (add-to-score! -1))
		  (and (> start-slot 5)
		       (< start-slot 13)
		       (not (empty-slot? start-slot))
		       (make-visible-top-card start-slot))
		  #t)
	      (move-n-cards! start-slot end-slot card-list)))
	(#t #f)))

(define (button-clicked slot-id)
  (if (= slot-id 0)
      (cond ((> (length (get-cards slot-id)) 7)
	     (and (deal-cards-face-up 0 reserve)
		  (give-status-message)))
	    ((> (length (get-cards slot-id)) 1)
	     (and (deal-cards-face-up 0 '(1))
		  (make-visible-top-card 0)
		  (give-status-message)))
	    (#t #f))
      #f))

(define (move-to-foundations? card slot-id)
  (cond ((> slot-id 5)
	 #f)
	((next-foundation-card? card slot-id)
	 (add-card! slot-id card))
	(#t
	 (move-to-foundations? card (+ 1 slot-id)))))

(define (button-double-clicked slot-id)
	(and
		(not (empty-slot? slot-id))
		(not (foundation? slot-id))
		(is-visible? (get-top-card slot-id))
		(move-to-foundations? (get-top-card slot-id) 2)
		(remove-card slot-id)
		(add-to-score! 1)
		(or
			(empty-slot? slot-id)
			(not (member slot-id tableau))
			(make-visible-top-card slot-id)
		)
	)
)

(define (game-continuable)
  (not (game-won)))

(define (game-won)
	(foundations-full?)
)

(define (check-to-tableaus slot-id)
  (cond ((>slot-id 19)
	 #f)
	((= slot-id 2)
	 (check-to-tableaus 6))
	((or (empty-slot? slot-id) 
	     (not (is-visible? (get-top-card slot-id))))
	 (check-to-tableaus (+ 1 slot-id)))
	(#t
	 (check-to-tableaus (+ 1 slot-id)))))

(define (get-hint)
  (or (or-map check-to-foundation (append (list 0 1) tableau reserve))
      (deal-hint)))

(define (get-options) 
  #f)

(define (apply-options options) 
  #f)

(define (timeout) 
  #f)

(set-lambda new-game button-pressed button-released button-clicked
button-double-clicked game-continuable game-won get-hint get-options
apply-options timeout)
