; AisleRiot - neighbor.scm
; Copyright (C) 1998 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
  (initialize-playing-area)
  (set-ace-low)
  (make-standard-deck)
  (shuffle-deck)
 
	(add-cards)
	(set! stock 0)
	(set! tableau '(1 2 3 4 5 6 7 8 9 10 11 12 13 14 15
		16 17 18 19 20 21 22 23 24 25))
  
  (deal-cards-face-up stock tableau)

  (give-status-message)

  (list 7 5)
)

(define (add-cards)
	(define (add-row)
		(add-blank-slot)
		(add-normal-slot '())
		(add-normal-slot '())
		(add-normal-slot '())
		(add-normal-slot '())
		(add-normal-slot '())
		(add-carriage-return-slot)
	)
	(add-normal-slot DECK) 
	(add-row)
	(add-blank-slot)
	(add-row)
	(add-blank-slot)
	(add-row)
	(add-blank-slot)
	(add-row)
	(add-blank-slot)
	(add-row)
)

(define (give-status-message)
	(set-statusbar-message (get-stock-no-string))
)

(define (button-pressed slot-id card-list)
	(and
		(not (= stock slot-id))
		(not (empty-slot? slot-id))
	)
)

(define (to-tableau? card-list end-slot)
	(define val1 (get-value (car card-list)))
	(if (empty-slot? end-slot)
		(= king val1)
		(= 13 (+ val1 (get-value (get-top-card end-slot))))
	)
)

(define (get-neighbour-slots slot)
	(define left (list (+ -6 slot) (1- slot) (+ 4 slot)))
	(define right (list (+ -4 slot) (1+ slot) (+ 6 slot)))
	(define top (list (+ -4 slot) (+ -5 slot) (+ -6 slot)))
	(define bottom (list (+ 4 slot) (+ 5 slot) (+ 6 slot)))
	(cond
		((= slot 1) '(2 6 7))
		((= slot 5) '(4 9 10))
		((= slot 21) '(16 17 22))
		((= slot 25) '(19 20 24))
		((= 1 (modulo slot 5))
			(cons (+ -5 slot) (cons (+ 5 slot) right)))
		((= 0 (modulo slot 5))
			(cons (+ -5 slot) (cons (+ 5 slot) left)))
		((> 5 slot)
			(cons (1- slot) (cons (1+ slot) bottom)))
		((< 21 slot)
			(cons (1- slot) (cons (1+ slot) top)))
		(else (cons (+ -5 slot) (cons (+ 5 slot) (append left right))))
	)
)

(define (bunch-up to from)
	(if (< 25 from)
		to
		(if (empty-slot? from)
			(bunch-up to (1+ from))
			(begin
				(deal-cards-face-up from (list to))
				(bunch-up (1+ to) (1+ from))
			)
		)
	)
)

(define (fill-up slot)
	(if (or (< 25 slot) (empty-slot? stock))
		#t
		(begin
			(deal-cards-face-up stock (list slot))
			(fill-up (1+ slot))
		)
	)
)

(define (fill-tableau slot)
	(fill-up (bunch-up slot (1+ slot)))
)

(define (button-released start-slot card-list end-slot)
	(if (= (get-value (car card-list)) king)
		(and
			(= end-slot start-slot)
			(add-to-score! 1)
			(fill-tableau start-slot)
		)
		(and
			(member start-slot (get-neighbour-slots end-slot))
			(to-tableau? card-list end-slot)
			(add-to-score! 2)
			(remove-card end-slot)
			(fill-tableau (min start-slot end-slot))
		)
	)
)
	 
(define (button-clicked slot-id)
	(and
		(not (empty-slot? slot-id))
		(= (get-value (get-top-card slot-id)) king)
		(add-to-score! 1)
		(remove-card slot-id)
		(fill-tableau slot-id)
	)
)

(define (button-double-clicked slot)
  #f)     

(define (game-won)
	(empty-slot? 1)
)

(define (game-over)
  (give-status-message)
  (and (not (game-won))
       (get-hint)))

(define (king-check slot-id)
	(and
		(not (empty-slot? slot-id))
  		(= king (get-value (get-top-card slot-id)))
		(list 2 (get-name (get-top-card slot-id)) "itself")
	)
)

(define (check-tableau slot)
	(define (f dest)
		(and
			(to-tableau? (get-cards slot) dest)
			(list 1 (get-name (get-top-card slot))
				(get-name (get-top-card dest)))
		)
	)
	(and
		(not (empty-slot? slot))
		(not (= king (get-value (get-top-card slot))))
		(or-map f (get-neighbour-slots slot))
	)
)

(define (get-hint)
	(or
		(or-map check-tableau tableau)
		(or-map king-check tableau)
	)
)

(define (get-options) #f)

(define (apply-options options) #f)

(define (timeout) #f)

(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
