; AisleRiot - odessa.scm
; Copyright (C) 1998 Rosanna Yuen <rwsy@mit.edu>
;                    Felix Bellaby <felix@pooh.u-net.com>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA
;
; try game 1433355966

(define (new-game) 
  (initialize-playing-area)
  (set-ace-low)
  (make-standard-deck)
  (shuffle-deck)
  
  (add-normal-slot DECK)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-carriage-return-slot)
  (add-normal-slot '())
  (add-carriage-return-slot)
  (add-normal-slot '())
  (add-carriage-return-slot)
  (add-normal-slot '())
  
  (set! foundations '(0 8 9 10))
  (set! tableau '(1 2 3 4 5 6 7))

  (deal-cards 0 (append tableau tableau tableau))
  (deal-cards-face-up 0 (append tableau tableau tableau '(2 3 4 5 6 2 3 4 5 6)))

  (map (lambda (x) (begin-score (reverse (get-cards x)) #f)) tableau)

  (list 8 4)
)

(define (begin-score card-list first-run?)
	(if (is-visible? (car card-list))
		(if (suit-next? (cadr card-list) (car card-list))
			(if first-run?
				(add-to-score! 1)
				(begin
					(add-to-score! 2)
					(set! first-run? #t)
				)
			)
			(set! first-run? #f)
		)
	)
	(and
		(> (length card-list) 2)
		(begin-score (cdr card-list) first-run?)
	)
)

(define (button-pressed slot-id card-list)
	(and
		(not (empty-slot? slot-id))
		(member slot-id tableau)
		(is-visible? (car (reverse card-list)))
	)
)

(define (num-in-a-row card rest)
	(define (f card rest n)
		(if
			(and
				(not (null? rest))
				(or
					(suit-next? card (car rest))
					(suit-next? (car rest) card)
				)
			)
			(f (car rest) (cdr rest) (+ 1 n))
			n
		)
	)
	(f card rest 0)
)

(define (complete-transaction start-slot card-list rcards end-slot)
	(and
		(member end-slot tableau)
		;prevents earning easy points moving kings!
		(not (empty-slot? end-slot))
		(let ((cards (get-cards end-slot)))
		      	(add-to-score! (+ 1
				(num-in-a-row (car cards) (cdr cards))
				(num-in-a-row (car cards) rcards)
			))
		)
	)
	(if (foundation? end-slot)
		(add-to-score! (length card-list)))
	(move-n-cards! start-slot end-slot card-list)
	(if (not (empty-slot? start-slot))
		(make-visible-top-card start-slot))
	#t
)

(define (button-released start-slot cards end-slot)
	(define rcards (reverse cards))
	(and
		(not (= start-slot end-slot))
		(if (member end-slot tableau)
			(and
				(if (empty-slot? end-slot)
					(= king (get-value (car rcards)))
					(suit-next? (car rcards) (get-top-card end-slot))
				)
				(complete-transaction start-slot cards rcards end-slot)
			)
			(and
				(next-foundation-card? (car cards) end-slot)
				(check-same-suit-list cards)
				(check-straight-descending-list cards)
				(complete-transaction start-slot rcards cards end-slot)
			)
		)
	)
)

(define (button-clicked slot)
	#f
)

(define (button-double-clicked slot)
	(and
		(member slot tableau)
		(not (empty-slot? slot))
		(let ((dest (find-foundation-slot (get-top-card slot))))
			(and
				dest
				(add-to-score! 1)
				(move-n-cards! slot dest (list (remove-card slot)))
			)
		)
		(if (not (empty-slot? slot))
			(make-visible-top-card slot))
	)
)

(define (is-visible-card cards card)
	(and
		(not (null? cards))
		(if (suit-next? (car cards) card)
			(and
				(is-visible? (car cards))
				(list 1 (get-name (car cards)) (get-name card))
			)
			(is-visible-card (cdr cards) card)
		)
	)
)

(define (check-to-tableau slot-id)
	(define (f slot2)
		(define card (get-top-card slot-id))
		(and
			(not (= slot-id slot2))
			(not (empty-slot? slot2))
			(is-visible-card (get-cards slot2) card)
		)
	)
	(and
		(not (empty-slot? slot-id))
		(or-map f tableau)
	)
)

(define (game-over)
	(get-hint)
)

(define (game-won)
	(foundations-full?)
)

(define (get-hint)
	(or
		(and
			(or-map empty-slot? tableau)
			(or-map here-kingy-kingy tableau)
		)
		(or-map check-to-tableau tableau)
		(or-map check-to-foundation tableau)
	)
)

(define (get-options) #f)

(define (apply-options options) #f)

(define (timeout) #f)

(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
