; AisleRiot - osmosis.scm
; Copyright (C) 1998 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA


(define (new-game)
	(initialize-playing-area)
	(set-ace-low)
	(make-standard-deck)
	(shuffle-deck)
	(set-redeals 2)

	(add-slots)

	(set! tableau '(0 2 4 6))
	(set! foundations '(1 3 5 7))
	(set! stock 8)
	(set! waste 9)
	(set! moveable (cons waste tableau))

	(deal-cards 8 (append tableau tableau tableau))
	(deal-cards-face-up 8 (append tableau (list 1)))

	(set-base-card! (get-value (get-top-card 1)))

	(add-to-score! 1)
	(give-status-message)

	(list 6 5)
)

(define (add-slots)
	(define (add-row)
		(add-extended-slot '() right)
		(add-blank-slot)
		(add-extended-slot '() right)
		(add-carriage-return-slot)
	)

	(add-row)
	(add-row)
	(add-row)
	(add-row)
	(add-normal-slot DECK)
	(add-normal-slot '())
)

(define (give-status-message)
	(set-statusbar-message (string-append (get-stock-no-string) "   "
		(get-redeals-string)))
)

(define (next-foundation-card? card slot)
	(if (empty-slot? slot)
		(and
			(= BASE (get-value card))
			(not (empty-slot? (- slot 2)))
		)
		(and
			(suit-eq? card (get-top-card slot))
			(or
				(= slot 1)
				(or-map (lambda (c) (value-eq? card c))
					(get-cards (- slot 2)))
			)
		)
	)
)

(define (button-pressed slot-id card-list)
	(and
		(not (empty-slot? slot-id))
		(= (length card-list) 1)
		(member slot-id moveable)
	)
)
		
(define (complete-transaction start-slot card-list end-slot)
	(move-n-cards! start-slot end-slot card-list)
	(add-to-score! 1)
	(if (not (empty-slot? start-slot))
		(make-visible-top-card start-slot)
	)
	#t
)

(define (button-released start-slot card-list end-slot)
	(and
		(not (= start-slot end-slot))
		(foundation? end-slot)
		(next-foundation-card? (car card-list) end-slot)
		(complete-transaction start-slot card-list end-slot)
	)
)
  
(define (button-clicked slot-id)
	(and
		(= slot-id stock)
		(flip-stock)
	)
)

(define (button-double-clicked slot)
	(and
		(member slot moveable)
		(not (empty-slot? slot))
		(let	((dest (find-foundation-slot (get-top-card slot))))
			(and
				dest
				(complete-transaction slot
					(list (remove-card slot)) dest)
			)
		)
	)
)

(define (game-over)
	(give-status-message)
	(get-hint)
)

(define (game-won)
	(foundations-full?)
)

(define (get-hint)
	(or
		(or-map check-to-foundation moveable)
		(deal-hint)
	)
)

(define (get-options) #f)

(define (apply-options options) #f)

(define (timeout) #f)

(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
