; AisleRiot - spider.scm
; Copyright (C) 1998 Jonathan Blandford <jrb@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
	(initialize-playing-area)
	(make-standard-double-deck)
	(shuffle-deck)
	(set-ace-low)
	(set-deal-count 10)

	(set! stock 0)
	(set! foundations '(1 2 3 4 5 6 7 8))
	(set! tableau '(9 10 11 12 13 14 15 16 17 18))

	(add-slots)

	(deal-cards 0 (append tableau tableau tableau tableau tableau
		'(9 12 15 18)))
	(map flip-top-card tableau)

	(give-status-message)

	(list 10 4)
)

(define (give-status-message)
  (set-statusbar-message (get-stock-no-string)))

;internal procedures/variables

(define (add-slots)
  (add-normal-slot DECK)
  (add-blank-slot)

	(do
		((i 0 (+ 1 i)))
		((= 8 i))
		(add-normal-slot '())
	)
	(add-carriage-return-slot)
	(do
		((i 0 (+ 1 i)))
		((= 10 i))
		(add-extended-slot '() down)
	)
)

;additional functions.

(define (complete-transaction start-slot card-list end-slot)
  (if (and (not (empty-slot? start-slot))
	   (is-visible? (get-top-card start-slot))
	   (suit-next? (car (reverse card-list)) (get-top-card start-slot)))
      (add-to-score! -1))
  (if (and (not (empty-slot? end-slot))
	   (is-visible? (get-top-card end-slot))
	   (suit-next? (car (reverse card-list)) (get-top-card end-slot)))
      (add-to-score! 1))
  (move-n-cards! start-slot end-slot card-list)
  (if (and (not (empty-slot? start-slot)) (> start-slot 5))
      (make-visible-top-card start-slot)
      #f)
  #t)

(define (check-for-points slot-id)
	(and
		(is-visible? (cadr (get-cards slot-id)))
		(suit-next? (get-top-card slot-id) (cadr (get-cards slot-id)))
		(add-to-score! 1)
	)
)

(define (deal-new-cards)
	(and
		(not (empty-slot? stock))
		(deal-cards-face-up stock tableau)
		(map check-for-points tableau)
	)
)

(define (button-pressed slot card-list)
	(give-status-message)
	(and
  		(not (empty-slot? slot))
		(member slot tableau)
		(not (null? card-list))
		(is-visible? (car (reverse card-list)))
		(check-same-suit-list card-list)
		(check-straight-descending-list card-list)
	)
)

(define (button-released start-slot card-list end-slot)
	(and
		(not (= start-slot end-slot))
		(if (empty-slot? end-slot)
			(cond
				((and
					(foundation? end-slot) 
					(= 13 (length card-list)))
				 (complete-transaction start-slot card-list end-slot))
				((> end-slot 8)
					(complete-transaction start-slot card-list end-slot))
				(#t #f)
			)
			(and
				(> end-slot 8)
				(card-follows? (car (reverse card-list))
					(get-top-card end-slot))
				(complete-transaction start-slot card-list end-slot)
			)
		)
	)
)

(define (button-clicked slot)
	(and
		(= 0 slot)
		(if (find-empty-slot tableau)
			(
				(set-statusbar-message "Please fill in empty pile first.")
				#f
			)
			(begin
			  	(deal-new-cards)
				(give-status-message)
				#t
			)
		)
	)
)

(define (button-double-clicked slot)
  #f)

(define (game-over)
  (and (not (game-won))
       (get-hint)))

(define (game-won)
	(foundations-full?)
)

(define (depth-card card-list)
  (if (and (> (length card-list) 1)
	   (is-visible? (cadr card-list))
	   (suit-next? (car card-list) (cadr card-list)))
      (depth-card (cdr card-list))
      card-list))

(define (check-a-slot slot1 card-to-move slot2 same-suit?)
  (if (> slot2 18)
      #f
      (if (and (not (= slot1 slot2))
	       (not (empty-slot? slot2))
	       (eq? same-suit? (suit-eq? card-to-move (get-top-card slot2)))
	       (card-follows? card-to-move (get-top-card slot2)))
	  (list 1
		(get-name card-to-move)
		(get-name (get-top-card slot2)))
	  (check-a-slot slot1 card-to-move (+ 1 slot2) same-suit?))))

(define (same-suit-check slot-id)
	(and
		(not (empty-slot? slot-id))
		(check-a-slot slot-id (car (depth-card (get-cards slot-id))) 9 #t)
	)
)

(define (not-same-suit-check slot-id)
	(and
		(not (empty-slot? slot-id))
		(or
			(= 1 (length (depth-card (get-cards slot-id))))
			(not (card-follows?
				(car (depth-card (get-cards slot-id)))
				(cadr (depth-card (get-cards slot-id))))
			)
		)
		(check-a-slot slot-id (car (depth-card (get-cards slot-id))) 9 #f)
	)
)

(define (open-slots?)
	(and
		(find-empty-slot tableau)
		(list 0 "Place something on empty slot")
	)
)

(define (get-hint)
  (or (or-map same-suit-check tableau)
      (or-map not-same-suit-check tableau)
      (open-slots?)
      (deal-hint)))

(define (get-options) #f)

(define (apply-options options) #f)

(define (timeout) #f)

(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
