; AisleRiot - straight_up.scm
; Copyright (C) 1999 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
	(initialize-playing-area)
	(set-ace-high)
	(make-restricted-deck 3)
	(shuffle-deck)
	(set-redeals 2)

  (add-normal-slot DECK)
  (add-normal-slot '())

  (add-blank-slot)

  (add-normal-slot (list (make-visible-card 2 club)))
  (add-normal-slot (list (make-visible-card 2 diamond)))
  (add-normal-slot (list (make-visible-card 2 heart)))
  (add-normal-slot (list (make-visible-card 2 spade)))

  (add-carriage-return-slot)

  (add-normal-slot '())

  (add-blank-slot)
  (add-blank-slot)

  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)

  (set! foundations '(2 3 4 5))
  (set! tableau '(7 8 9 10))
  (set! reserve 6)
  (set! stock 0)
  (set! waste 1)
  (set! moveable (cons waste (cons reserve tableau)))

  (deal-cards 0 '(6 6 6 6 6 6 6 6 6 6 6 6))
  (deal-cards-face-up 0 '(6 7 8 9 10))

  (give-status-message)

  (list 7 3)
)

(define (give-status-message)
	(set-statusbar-message (string-append
		(get-stock-no-string) "   "
		(get-reserve-no-string) "   "
		(get-redeals-string)
	))
)

(define (button-pressed slot-id card-list)
	(and
		(not (empty-slot? slot-id))
		(member slot-id moveable)
	)
)

(define (button-released start-slot card-list end-slot)
  (cond ((foundation? end-slot)
	 (and (suit-next? (get-top-card end-slot) (car card-list))
	      (add-to-score! (length card-list))
	      (move-n-cards! start-slot end-slot (reverse card-list))
	      (check-reserve start-slot)))
	((member end-slot tableau)
	 (if (empty-slot? end-slot)
	     (and (empty-slot? reserve)
		  (= start-slot 1)
		  (move-n-cards! start-slot end-slot card-list))
	     (and (suit-next? (car (reverse card-list)) (get-top-card end-slot))
		  (move-n-cards! start-slot end-slot card-list)
		  (check-reserve start-slot))))
	(#t #f))
)

(define (check-reserve start-slot)
  (or (< start-slot reserve)
      (empty-slot? reserve)
      (and (= reserve start-slot)
	   (make-visible-top-card reserve)
	   (give-status-message))
      (not (empty-slot? start-slot))
      (and (add-card! start-slot (remove-card reserve))
	   (or (empty-slot? reserve)
	       (make-visible-top-card reserve))
	   (give-status-message)))
	#t
)

(define (button-clicked slot-id)
	(and
		(= slot-id 0)
		(flip-stock)
		(give-status-message)
	)
)

(define (button-double-clicked slot-id)
	(and
		(not (empty-slot? slot-id))
		(is-visible? (get-top-card slot-id))
		(let* (
			(card (get-top-card slot-id))
			(dest (find-foundation-slot card))
		)
			(and
				dest
				(move-n-cards! slot-id dest (list card))
				(add-to-score! 1)
				(remove-card slot-id)
				(check-reserve slot-id)
			)
		)
	)
)

(define (game-continuable)
  (and (not (game-won))
       (get-hint)))

(define (game-won)
	(foundations-full?)
)

(define (check-a-tableau card t-slot)
	(and
		(not (empty-slot? t-slot))
		(suit-next? card (get-top-card t-slot))
		(list 1 (get-name card) (get-name (get-top-card t-slot)))
	)
)

(define (to-tableau slot-id)
	(define (f dest)
		(check-a-tableau
			(if (member slot-id tableau)
				(get-bottom-card slot-id)
				(get-top-card slot-id)
			) dest
		)
	)
	(and
		(not (empty-slot? slot-id))
		(or-map f tableau)
	)
)

(define (empty-tableau? slot-id)
  (cond ((or (empty-slot? 1)
	     (> slot-id 10))
	 #f)
	((empty-slot? slot-id)
	 (list 2 (get-name (get-top-card 1)) "an empty Tableau slot"))
	(#t (empty-tableau? (+ 1 slot-id)))))

(define (get-hint)
	(or
		(or-map check-to-foundation moveable)
		(or-map to-tableau moveable)
		(empty-tableau? 7)
		(deal-hint)
	)
)

(define (get-options) 
  #f)

(define (apply-options options) 
  #f)

(define (timeout) 
  #f)

(set-lambda new-game button-pressed button-released button-clicked
button-double-clicked game-continuable game-won get-hint get-options
apply-options timeout)
