; AisleRiot - thumb_and_pouch.scm
; Copyright (C) 1999 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
	(initialize-playing-area)
	(set-ace-low)
	(make-standard-deck)
	(shuffle-deck)
	(set-redeals 2)
  
  (add-normal-slot DECK)

  (add-normal-slot '())

  (add-blank-slot)
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (add-carriage-return-slot)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)

	(set! stock 0)
	(set! waste 1)
	(set! foundations '(2 3 4 5))
	(set! tableau '(6 7 8 9 10 11 12))

  (deal-cards 0 '(6 7 8 9 10 11 12 7 8 9 10 11 12 8 9 10 11 12 9 10 11 12 10 11 12 11 12 12))

  (map flip-top-card tableau)

  (give-status-message)

  (list 7 3))

(define (give-status-message)
  (set-statusbar-message (string-append (get-stock-no-string)
					"   "
					(get-redeals-string))))

(define (button-pressed slot-id card-list)
	(and
		(> slot-id 0)
		(is-visible? (car (reverse card-list)))
	)
)

(define (complete-transaction start-slot card-list end-slot)
	(move-n-cards! start-slot end-slot card-list)
	(if (foundation? start-slot)
		(add-to-score! -1))
	(if (foundation? end-slot)
		(add-to-score! 1))
	(and
		(not (empty-slot? start-slot))
		(member start-slot tableau)
		(make-visible-top-card start-slot)
	)
	#t
)

(define (tableau-next? first second)
	(and
		(not (suit-eq? first second))
		(card-follows? first second)
	)
)

(define (next-tableau-card? card slot)
	(or
		(empty-slot? slot)
		(tableau-next? card (get-top-card slot))
	)
)

(define (button-released start-slot card-list end-slot)
  (and (not (= start-slot end-slot))
       (or (and (member end-slot tableau)
		(next-tableau-card? (car (reverse card-list)) end-slot))
	   (and (foundation? end-slot)
		(= 1 (length card-list))
		(next-foundation-card? (car card-list) end-slot)))
       (complete-transaction start-slot card-list end-slot))
)

(define (button-clicked slot-id)
	(and
		(= slot-id 0)
		(flip-stock)
		(give-status-message)
	)
)

(define (button-double-clicked slot)
	(and
		(member slot (cons 1 tableau))
		(not (empty-slot? slot))
		(let ((dest (find-foundation-slot (get-top-card slot))))
			(and
				dest
				(complete-transaction slot
					(list (remove-card slot)) dest)
			)
		)
	)
)

(define (game-continuable)
	(and
		(not (game-won))
		(get-hint)
	)
)

(define (game-won)
	(define (f slot)
		(or
			(empty-slot? slot)
			(is-visible? (get-bottom-card slot))
		)
	)
	(and
		(empty-slot? 0)
		(empty-slot? 1)
		(and-map f tableau)
	)
)

(define (check-waste-to-empty)
  (and (not (empty-slot? 1))
       (list 2 (get-name (get-top-card 1)) "an empty Tableau pile")))

(define (stripped card-list)
  (if (not (is-visible? (cadr card-list)))
      (car card-list)
      (stripped (cdr card-list))))

(define (check-tableau-to-empty slot-id)
	(and
		(not (empty-slot? slot-id))
		(not (is-visible? (get-bottom-card slot-id)))
		(list 2 (get-name (stripped (get-cards slot-id)))
			"an empty Tableau pile")
	)
)

(define (check-empty?)
	(and
		(or-map empty-slot? tableau)
		(or
			(or-map check-tableau-to-empty tableau)
			(check-waste-to-empty)
		)
	)
)

(define (check-a-tslot card card-list)
	(and
		(not (= (length card-list) 0))
		(is-visible? (car card-list))
		(< (get-value (car card-list)) (get-value card))
		(or
			(and
				(tableau-next? (car card-list) card)
				(or
					(= (length card-list) 1)
					(not (is-visible? (cadr card-list)))
					(find-foundation-slot (cadr card-list))
				)
				(list 1 (get-name (car card-list)) (get-name card))
			)
			(check-a-tslot card (cdr card-list))
		)
	)
)

(define (check-to-tableau slot-id)
	(define (f slot2)
		(check-a-tslot (get-top-card slot-id) (get-cards slot2))
	)
	(and
		(not (empty-slot? slot-id))
		(or
			(and
				(not (empty-slot? 1))
				(check-a-tslot (get-top-card slot-id)
						(list (get-top-card 1)))
			)
			(or-map f tableau)
		)
	)
)

(define (get-hint)
	(or
		(or-map check-to-foundation (cons 1 tableau))
		(or-map check-to-tableau tableau)
		(check-empty?)
		(deal-hint)
	)
)

(define (get-options) 
  #f)

(define (apply-options options) 
  #f)

(define (timeout) 
  #f)

(set-lambda new-game button-pressed button-released button-clicked
button-double-clicked game-continuable game-won get-hint get-options
apply-options timeout)
