; AisleRiot - union_square.scm
; Copyright (C) 1999 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
  (initialize-playing-area)
  (set-ace-low)
  (make-standard-double-deck)
  (shuffle-deck)

  (add-normal-slot DECK)
  (add-normal-slot '())

  (add-blank-slot)

  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())

  (add-blank-slot)

  (add-normal-slot '())

  (add-carriage-return-slot)
  (add-blank-slot)
  (add-blank-slot)
  (add-blank-slot)

  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())

  (add-blank-slot)

  (add-normal-slot '())

  (add-carriage-return-slot)
  (add-blank-slot)
  (add-blank-slot)
  (add-blank-slot)

  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())

  (add-blank-slot)

  (add-normal-slot '())
  (add-carriage-return-slot)
  (add-blank-slot)
  (add-blank-slot)
  (add-blank-slot)

  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())

  (add-blank-slot)

  (add-normal-slot '())

	(set! foundations '(6 11 16 21))
	(set! tableau '(2 3 4 5  7 8 9 10  12 13 14 15  17 18 19 20))
	(set! stock 0)
	(set! waste 1)

  (deal-cards-face-up stock tableau)

  (give-status-message)

  (list 9 4)
)

(define (give-status-message)
  (set-statusbar-message (get-stock-no-string)))

(define (button-pressed slot-id card-list)
	(and
		(not (empty-slot? slot-id))
		(member slot-id (cons waste tableau))
	)
)

(define (next-foundation-card? card end-slot)
	(if (empty-slot? end-slot)
		(and
			(eq? (get-value card) ace)
			(not (find-slot-by-suit (get-suit card) foundations))
		)
		(and
			(suit-eq? card (get-top-card end-slot))
			(cond
				((< (length (get-cards end-slot)) 13)
				 (card-follows? (get-top-card end-slot) card))
				((= (length (get-cards end-slot)) 13)
				 (= (get-value card) king))
				(else
				 (card-follows? card (get-top-card end-slot)))
			)
		)
	)
)

(define (to-tableau? card end-slot)
	(define top (get-top-card end-slot))
	(or
		(empty-slot? end-slot)
		(and
			(suit-eq? card top)
			(or
				(card-follows? card top)
				(card-follows? top card)
			)
			(or
				(= 1 (length (get-cards end-slot)))
				(not (value-eq? card (cadr (get-cards end-slot))))
			)
		)
	)
)

(define (cards-to-tableau? card-list slot)
	(and
		(to-tableau? (car card-list) slot)
		(or
			(= 1 (length card-list))
			(not (value-eq? (cadr card-list) (get-top-card slot)))
		)
	)
)

(define (button-released start-slot card-list end-slot)
	(or
		(and
			(foundation? end-slot)
	 		(next-foundation-card? (car card-list) end-slot)
			(move-n-cards! start-slot end-slot card-list)
			(add-to-score! 1)
		)
		(and
			(member end-slot tableau)
			(cards-to-tableau? card-list end-slot)
			(move-n-cards! start-slot end-slot card-list)
		)
	)
)

(define (button-clicked slot-id)
	(and
		(= slot-id 0)
		(flip-stock)
	)
)

(define (button-double-clicked slot-id)
  #f)

(define (game-continuable)
  (give-status-message)
  (not (game-won)))

(define (game-won)
	(define (foundation-full? slot)
		(= 26 (length (get-cards slot)))
	)
	(and-map foundation-full? foundations)
)

(define (check-imbedded card-list foundation-id)
	(define (f card)
		(next-foundation-card? card foundation-id)
	)
	(or-map f card-list)
)

(define (check-slot-contents slot-id)
	(define found-slot
		(and	(not (empty-slot? slot-id))
			(find-slot-by-suit (get-suit (get-top-card slot-id))
				foundations)
		)
	)
	(define (f slot)
		(and	(not (= slot-id slot))
			(not (empty-slot? slot))
			(cards-to-tableau? (reverse (get-cards slot-id)) slot)
		)

	)
	(or
		(and	found-slot
			(check-imbedded (get-cards slot-id) found-slot)
			slot-id
		)
		(and
			(> (length (get-cards slot-id)) 1)
			(or-map f tableau)
			slot-id
		)
	)
)

(define (check-a-tslot start)
	(define card-list
		(if (= 1 start)
			(list (get-top-card 1))
			(get-cards start)
		)
	)
	(define (f end)
		(and
			(not (empty-slot? end))
			(not (= start end))
			(cards-to-tableau? card-list end)
			(list 1 (get-name (car card-list))
				(get-name (get-top-card end)))
		)
	)
	(and
		(not (empty-slot? start))
		(or-map f tableau)
	)
)

(define (check-tableau)
	(define (f slot)
		(check-a-tslot slot)
	)
	(or
		(or-map f tableau)
		(f 1)
	)
)

(define (check-rev-tableau start)
	(define card-list (reverse (get-cards start)))
	(define (f end)
		(and
			(not (empty-slot? end))
			(not (= start end))
			(cards-to-tableau? card-list end)
		)
	)
	(or-map f tableau)
)

(define (check-for-bottom)
	(define (f slot)
		(and
			(not (empty-slot? slot))
			(check-rev-tableau slot)
			(list 2 (get-name (get-top-card slot)) "an empty slot")
		)
	)
	(or
		(or-map f tableau)
	)
)

(define (contents-check)
	(define (f slot)
		(and
			(check-slot-contents slot)
	     		(list 2 (get-name (get-top-card slot)) "an empty slot")
		)
	)
	(or-map f tableau)
)

(define (check-empty-slot)
  (and (find-empty-slot tableau)
      (cond ((contents-check))
	    ((check-for-bottom))
	    ((not (empty-slot? 1))
	     (list 2 (get-name (get-top-card 1)) "an empty slot"))
	    (#t #f))))

(define (get-hint)
	(or
		(or-map check-to-foundation tableau)
		(check-tableau)
		(check-empty-slot)
		(deal-hint)
	)
)

(define (get-options) 
  #f)

(define (apply-options options) 
  #f)

(define (timeout) 
  #f)

(set-lambda new-game button-pressed button-released button-clicked
button-double-clicked game-continuable game-won get-hint get-options
apply-options timeout)
