; AisleRiot - westhaven.scm
; Copyright (C) 1999 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
	(initialize-playing-area)
	(set-ace-low)
	(make-standard-deck)
	(shuffle-deck)
	(set-redeals 0)

  (add-normal-slot DECK)
  (add-normal-slot '())

  (add-blank-slot)

  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())
  (add-normal-slot '())

  (add-carriage-return-slot)

  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)

	(set! stock 0)
	(set! waste 1)
	(set! foundations '(2 3 4 5))
	(set! tableau '(6 7 8 9 10 11 12 13 14 15))
	(deal-cards 0 (append tableau tableau tableau))
	(map flip-top-card tableau)

  (give-status-message)

  (list 10 4)
)

(define (give-status-message)
  (set-statusbar-message (get-stock-no-string)))

(define (next-card card-list number)
	(car (nthcdr number card-list))
)

(define (button-pressed slot-id card-list)
  (and (not (empty-slot? slot-id))
       (not (foundation? slot-id))
       (is-visible? (car (reverse card-list)))
       (or (= (length card-list) 1)
	   (= (length card-list)
	      (length (get-cards slot-id)))
	   (not (is-visible? (next-card (get-cards slot-id)
					(length card-list)))))))

(define (check-move card slot)
	(and
		(not (eq? (get-color (get-top-card slot)) (get-color card)))
		(card-follows? card (get-top-card slot))
	)
)

(define (to-tableau? card-list end-slot)
	(or
		(empty-slot? end-slot)
		(check-move (car (reverse card-list)) end-slot)
	)
)

(define (button-released start-slot card-list end-slot)
	(or
		(and
			(member end-slot tableau)
			(to-tableau? card-list end-slot)
			(move-cards-to-slot start-slot end-slot card-list)
		)
		(and
			(foundation? end-slot)
			(= 1 (length card-list))
			(next-foundation-card? (car card-list) end-slot)
			(add-to-score! 1)
			(move-cards-to-slot start-slot end-slot card-list)
		)
	)
)

(define (button-clicked slot-id)
	(and
		(= slot-id 0)
		(flip-stock)
	)
)

(define (move-cards-to-slot start-slot end-slot card-list)
	(move-n-cards! start-slot end-slot card-list)
	(or
		(empty-slot? start-slot)
		(make-visible-top-card start-slot)
	)
)

(define (button-double-clicked start-slot)
	(and
		(not (empty-slot? start-slot))
		(member start-slot (cons 1 tableau))
		(let* ( (card (get-top-card start-slot))
			(end-slot (find-foundation-slot card)) )
			(and
				end-slot
				(remove-card start-slot)
				(add-to-score! 1)
				(move-cards-to-slot
					start-slot end-slot (list card))
			)
		)
	)
)

(define (game-continuable)
  (give-status-message)
  (and (not (game-won))
       (get-hint)))

(define (game-won)
	(foundations-full?)
)

(define (to-foundations)
	(define (f slot)
		(and
			(not (empty-slot? slot))
			(let* ( (card (get-top-card slot))
				(end-slot (find-foundation-slot card)) )
				(and
					end-slot
					(if (empty-slot? end-slot)
						(list 2 (get-name card)
							"an empty Foundation")
						(list 1 (get-name card)
							(get-name (get-top-card
							end-slot)))
					)
				)
			)
		)
	)
	(or-map f (cons 1 tableau))
)

(define (waste-to-tableau)
	(define (f slot)
		(and
			(not (empty-slot? slot))
			(check-move (get-top-card 1) slot)
			(list 1 (get-name (get-top-card 1))
				(get-name (get-top-card slot)))
		)
	)
	(and
		(not (empty-slot? 1))
		(or-map f tableau)
	)
)

(define (get-available-bottom slot-id)
	(define (f card)
		(and (is-visible? card) card)
	)
	(or-map f (reverse (get-cards slot-id)))
)

(define (cards-to-tableau card)
	(define (f slot)
		(and
			(not (empty-slot? slot))
			(check-move card slot)
			slot
		)
	)
	(or-map f tableau)
)

(define (tableau-to-tableau)
	(define (f slot)
		(and
			(not (empty-slot? slot))
			(let* (
				(card (get-available-bottom slot))
				(end-slot (cards-to-tableau card)) )
				(and
					end-slot
					(list 1 (get-name card) (get-name
						(get-top-card end-slot)))
				)
			)
		)
	)
	(or-map f tableau)
)

(define (find-covered)
	(define (f slot)
		(and
			(not (empty-slot? slot))
			(not (is-visible? (car (reverse (get-cards slot)))))
			(list 2 (get-name (get-available-bottom slot))
				"an empty Tableau pile")
		)
	)
	(or-map f tableau)
)

(define (check-waste)
	(and
		(not (empty-slot? 1))
		(list 2 (get-name (get-top-card 1)) "an empty Tableau pile")
	)
)

(define (check-empty-slot)
	(and
		(find-empty-slot tableau)
		(or
			(find-covered)
			(check-waste)
		)
	)
)

(define (get-hint)
	(or
		(to-foundations)
		(waste-to-tableau)
		(tableau-to-tableau)
		(check-empty-slot)
		(deal-hint)
	)
)

(define (get-options) 
  #f)

(define (apply-options options) 
  #f)

(define (timeout) 
  #f)

(set-lambda new-game button-pressed button-released button-clicked
button-double-clicked game-continuable game-won get-hint get-options
apply-options timeout)
