; AisleRiot - yukon.scm
; Copyright (C) 1998 Rosanna Yuen <rwsy@mit.edu>
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA


(define (new-game)
  (initialize-playing-area)

  ;set up the cards
  (make-standard-deck)
  (shuffle-deck)
  
  ;set up the board
  (add-normal-slot DECK)
  (add-blank-slot)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-extended-slot '() down)
  (add-carriage-return-slot)
  (add-normal-slot '())
  (add-carriage-return-slot)
  (add-normal-slot '())
  (add-carriage-return-slot)
  (add-normal-slot '())

  (set! foundations '(0 8 9 10))
  (set! tableau '(1 2 3 4 5 6 7))

  (deal-cards 0
  	'(1 2 3 4 5 6 7
	    2 3 4 5 6 7
	      3 4 5 6 7
	        4 5 6 7
		  5 6 7
		    6 7
		      7))

  (map flip-top-card tableau)

  (deal-cards-face-up 0 '(2 3 4 5 6 7))
  (deal-cards-face-up 0 '(2 3 4 5 6 7))
  (deal-cards-face-up 0 '(2 3 4 5 6 7))
  (deal-cards-face-up 0 '(2 3 4 5 6 7))

  (list 9 4))

(define (button-pressed slot-id card-list)
	(and
		card-list
		(member slot-id tableau)
		(is-visible? (car (reverse card-list)))
	)
)
      
(define (complete-transaction start-slot card-list end-slot)
	(move-n-cards! start-slot end-slot card-list)
	(and
		(foundation? end-slot)
		(add-to-score! 1)
	)
	(if (not (empty-slot? start-slot))
		(make-visible-top-card start-slot)
	)
)

(define (next-tableau-card? card slot)
	(if (empty-slot? slot)
		(= king (get-value card))
		(and
			(not (color-eq? card (get-top-card slot)))
			(card-follows? card (get-top-card slot))
		)
	)
)

(define (button-released start-slot card-list end-slot)
	(if (foundation? end-slot)
		(and
			(= 1 (length card-list))
			(next-foundation-card? (car card-list) end-slot)
			(complete-transaction start-slot card-list end-slot)
		)
		(and
			(next-tableau-card? (car (reverse card-list)) end-slot)
			(complete-transaction start-slot card-list end-slot)
		)
	)
)

(define (button-clicked slot-id)
  #f)

(define (button-double-clicked slot)
	(and
		(not (empty-slot? slot))
		(member slot tableau)
		(let ((end-slot (find-foundation-slot (get-top-card slot))))
			(and
				end-slot
				(complete-transaction slot
					(list (remove-card slot)) end-slot)
			)
		)
	)
)

(define (game-over)
  (and (not (game-won))
       (get-hint)))

(define (game-won)
	(foundations-full?)
)

(define (check-for-empty)
	(and
		(or-map empty-slot? tableau)
		(or-map here-kingy-kingy tableau)
	)
)

(define (stripped card-list card)
	(if (< (length card-list) 2)
		'()
		(if (eq? card (car card-list))
			(cdr card-list)
			(stripped (cdr card-list) card)
		)
	)
)

(define (check-a-tableau card slot1 card-list slot2 imbedded?)
  (cond ((or (= (length card-list) 0)
	     (not (is-visible? (car card-list))))
	 #f)
	((and (not (color-eq? (car card-list) card))
		(card-follows? (car card-list) card))
	 (if (or  (= (length card-list) 1)
		  (color-eq? (car card-list) (cadr card-list))
		  imbedded?
		  (not (card-follows? (car card-list) (cadr card-list)))
		  (find-foundation-slot (cadr card-list))
		  (check-a-tableau (get-top-card slot2) slot1	
				   (cdr card-list) slot2 #t)
		  (check-a-tableau (cadr card-list) slot2
				   (get-cards slot1) slot1 #t)
		  (check-a-tableau (cadr card-list) slot2
				   (stripped (get-cards slot2) (car card-list))
				   slot2 #t))
	     (list 1 (get-name (car card-list)) (get-name card))
	     (and (not imbedded?)
		  (check-a-tableau card slot1 
				   (cdr card-list) slot2 imbedded?))))
	(imbedded? #f)
	(#t (check-a-tableau card slot1 (cdr card-list) slot2 imbedded?))))

(define (check-to-tableau slot1)
	(define (f slot2)
		(and
			(not (= slot1 slot2))
			(not (empty-slot? slot1))
			(check-a-tableau (get-top-card slot1) slot1
				(get-cards slot2) slot2 #f)
		)
	)
	(or-map f tableau)
)

(define (get-hint)
	(or
		(or-map check-to-foundation tableau)
		(or-map check-to-tableau tableau)
		(check-for-empty)
	)
)

(define (get-options) #f)

(define (apply-options options) #f)

(define (timeout) #f)

(set-lambda new-game button-pressed button-released button-clicked button-double-clicked game-over game-won get-hint get-options apply-options timeout)
