; AisleRiot - zebra.scm
; Copyright (C) 1999 Rosanna Yuen <rwsy@mit.edu>
;
; This game is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2, or (at your option)
; any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
; USA

(define (new-game)
	(initialize-playing-area)
	(set-ace-high)
	(make-restricted-double-deck 3)
	(set-redeals 1)
	(shuffle-deck)

	(add-slots)

	(set! stock 0)
	(set! waste 1)
	(set! foundations '(2 3 4 5 6 7 8 9))
	(set! tableau '(10 11 12 13 14 15 16 17))
	(set! moveable (cons waste tableau))

	(deal-cards-face-up stock tableau)

	(give-status-message)

	(list 10 3)
)

(define (add-slots)
	(define card-sliver-width (quotient (* 21 (get-card-width)) 58))
	(add-normal-slot DECK)
	(add-normal-slot '())

	(set! HORIZPOS (+ HORIZPOS (quotient (get-card-width) 3)))

	(do
		((i 0 (+ 1 i)))
		((= 8 i))
		(add-normal-slot (list (make-visible-card 2 (modulo i 4))))
	)

	(add-carriage-return-slot)

	(do
		((i 0 (+ 1 i)))
		((= 8 i))
		(add-extended-slot '() down)
		(set! HORIZPOS (+ HORIZPOS card-sliver-width))
	)
)

(define (give-status-message)
	(set-statusbar-message (string-append
		(get-stock-no-string) "   " (get-redeals-string)))
)

(define (next-foundation-card? card slot)
	(define top (get-top-card slot))
	(and
		(not (color-eq? top card))
		(card-follows? top card)
	)
)

(define (button-pressed slot-id card-list)
  (and (not (empty-slot? slot-id))
       (or (= slot-id 1)
	   (> slot-id 9))))

(define (empty-tableau slot)
  (or (not (empty-slot? slot))
      (= slot 1)
      (and (not (empty-slot? 1))
	   (deal-cards-face-up 1 (list slot)))
      (and (not (empty-slot? 0))
	   (deal-cards-face-up 0 (list slot))
	   (give-status-message))
      #t))

(define (button-released start-slot card-list end-slot)
	(if (foundation? end-slot)
		(and
			(next-foundation-card? (car card-list) end-slot)
			(move-n-cards! start-slot end-slot (reverse card-list))
			(add-to-score! (length card-list))
			(empty-tableau start-slot)
		)
		(and
			(member end-slot tableau)
			(= 1 (length card-list))
			(next-tableau-card? (car card-list) end-slot)
			(move-n-cards! start-slot end-slot card-list)
			(empty-tableau start-slot)
		)
	)
)

(define (button-clicked slot-id)
  (and (= slot-id 0)
       (flip-stock)
       (give-status-message)))

(define (button-double-clicked slot-id)
	(and
		(not (empty-slot? slot-id))
		(member slot-id moveable)
		(let ((dest (find-foundation-slot (get-top-card slot-id))))
			(and
				dest
				(move-n-cards! slot-id dest
					(list (remove-card slot-id)))
			)
		)
		(add-to-score! 1)
		(empty-tableau slot-id)
	)
)

(define (game-continuable)
  (get-hint))

(define (game-won)
	(foundations-full?)
)

(define (check-a-tableau slot1 card-list slot2)
  (cond ((= slot2 18)
	 #f)
	((and (not (= slot1 slot2))
	      (not (eq? (is-red? (car card-list))
			(is-red? (get-top-card slot2))))
	      (= (+ 1 (get-value (car card-list)))
		 (get-value (get-top-card slot2)))
	      (or (= slot1 1)
		  (= (length card-list) 1)
		  (check-a-tableau slot1 (cdr card-list) 10)))
	 (list 1
	       (get-name (car card-list))
	       (get-name (get-top-card slot2))))
	(#t 
	 (check-a-tableau slot1 card-list (+ 1 slot2)))))

(define (check-to-tableaus slot-id)
  (cond ((= slot-id 18)
	 #f)
	((= slot-id 2)
	 (check-to-tableaus 10))
	((and (not (empty-slot? slot-id))
	      (check-a-tableau slot-id (get-cards slot-id) 10))
	 (check-a-tableau slot-id (get-cards slot-id) 10))
	(#t (check-to-tableaus (+ 1 slot-id)))))

(define (get-hint)
	(or
		(or-map check-to-foundation moveable)
		(check-to-tableaus 1)
		(deal-hint)
	)
)

(define (get-options) 
  #f)

(define (apply-options options) 
  #f)

(define (timeout) 
  #f)

(set-lambda new-game button-pressed button-released button-clicked
button-double-clicked game-continuable game-won get-hint get-options
apply-options timeout)
